#ifndef MNC_h
#define MNC_h

#include <matvec/matvec.h>
#include <BK/Struct.h>
#include <BK/Vypocty/Uhly.h>
#include <BK/Vypocty/XYDS.h>
#include <BK/Vypocty/Diff.h>
using GNU_gama::Mat;
using GNU_gama::Vec;

typedef vector<vector<string> > VVS;
typedef vector<string>          VS;

// fce pro inicializaci vektoru mereni
Vec<> VecL_DU( VVS& smCB, SM& sm )
{ int m = smCB.size();
  Vec<> L(m);
  for(int i=0; i<m; i++)
  { VS m = smCB[i];
    if( m.size()==3 )   L(i+1) = sm[m[1]][m[2]].Hz - sm[m[1]][m[0]].Hz;
    if( m.size()==2 )   L(i+1) = sm[m[0]][m[1]].D;
  }
  return L;
}

Vec<> VecL_DS( VVS& smCB, SM& sm )
{ int m = smCB.size();
  Vec<> L(m);
  for(int i=0; i<m; i++)
  { VS m = smCB[i];
    if( m.size()==3 )   L(i+1) = sm[m[0]][m[1]].Hz;
    if( m.size()==2 )   L(i+1) = sm[m[0]][m[1]].D;
  }
  return L;
}

// fce pro vypocet kovariancni matice mereni
Mat<> MatQL( VVS& smCB, double& sgm_D, double& sgm_U )
{ int m = smCB.size();
  Mat<> QL(m,m);
  for(int i=0; i<m; i++)
  { VS m = smCB[i];
    if( m.size()==3 ) QL(i+1,i+1) = sgm_U*sgm_U;
    if( m.size()==2 ) QL(i+1,i+1) = sgm_D*sgm_D;
  }
  return QL;
}

// fce pro vypocet matice parcialnich derivaci ( delky, uhly -> souradnice )
Mat<> MatA( VVS& smCB, VS& XCB, SS& ss )
{ int m = smCB.size();
  int n = XCB.size()*2;
  Mat<> A(m,n);
  for( int i=0; i<m; i++ )
  { VS m = smCB[i];
    for( int j=0; j<n/2; j++ ) // porovnavam body
    { int x = 0;
      if( m[0]==XCB[j] ) x = 1;
      if( m[1]==XCB[j] ) x = 3;
      if( m.size()==3 ) if( m[2]==XCB[j] ) x = 5;
      for( int k=0; k<2; k++ )
      { if( x==0 & k==1 ) x--;
        if( m.size()==3 )  A(i+1,(j+1)*2-1+k) = DiffU(ss[m[0]],ss[m[1]],ss[m[2]],x+k);
        if( m.size()==2 )  A(i+1,(j+1)*2-1+k) = DiffD(ss[m[0]],ss[m[1]],x+k);
      }
    }
  }
  return A;
}

// fce pro vypocet matice parcialnich derivaci ( delky, smery -> souradnice, or. posuny )
Mat<> MatA_OP( VVS& smCB, VS& XCB, SS& ss )
{ int m = smCB.size();
  int n = XCB.size()*2;
  Mat<> A(m,n+4);
  for( int i=0; i<m; i++ )
  { VS m = smCB[i];
    for( int j=0; j<n/2; j++ ) // porovnavam body
    { int x = 0;
      if( m[0]==XCB[j] ) x = 1;
      if( m[1]==XCB[j] ) x = 3;
      for( int k=0; k<2; k++ )
      { if( x==0 & k==1 ) x--;
        if( m.size()==3 )
        { A(i+1,(j+1)*2-1+k) = DiffS(ss[m[0]],ss[m[1]],x+k);
          // or. posuny
          if( m[0]=="A" ) A(i+1,n+1) = -1;
          if( m[0]=="B" ) A(i+1,n+2) = -1;
          if( m[0]=="C" ) A(i+1,n+3) = -1;
          if( m[0]=="D" ) A(i+1,n+4) = -1;
        }
        if( m.size()==2 )  A(i+1,(j+1)*2-1+k) = DiffD(ss[m[0]],ss[m[1]],x+k);
        
      }
    }
  }
  return A;
}

// fce pro vypocet mereni ze souradnic
Vec<> VecL_SS( VVS& smCB, SS& ss )
{ int m = smCB.size();
  Vec<> L(m);
  for(int i=0; i<m; i++)
  { VS m = smCB[i];
    if( m.size()==3 )
    { double A = GetS(ss[m[1]],ss[m[2]]) - GetS(ss[m[1]],ss[m[0]]);
      L(i+1) = Gra(Rad(A));
    }
    if( m.size()==2 ) L(i+1) = GetD(ss[m[0]],ss[m[1]]);
  }
  return L;
}

// fce pro vypocet mereni ze souradnic a orientacnich posunu
Vec<> VecL_OP( VVS& smCB, SS& ss, vector<double>& sOP )
{ int m = smCB.size();
  Vec<> L(m);
  for(int i=0; i<m; i++)
  { VS m = smCB[i];
    if( m.size()==3 )
    { double A = GetS(ss[m[0]],ss[m[1]]);
      if( m[0]=="A" ) A -= sOP[0];
      if( m[0]=="B" ) A -= sOP[1];
      if( m[0]=="C" ) A -= sOP[2];
      if( m[0]=="D" ) A -= sOP[3];
      L(i+1) = Gra(A,GRA);
    }
    if( m.size()==2 ) L(i+1) = GetD(ss[m[0]],ss[m[1]]);
  }
  return L;
}

// vektor smerodatnych odchylek z kovariancni matice
Vec<> VecSgm( Mat<>& A )
{ int d = A.rows();
  Vec<> Sgm(d);
  for(int i=1; i<=d; i++)
    Sgm(i) = sqrt( A(i,i) );
  return Sgm;
}

#endif
